from njserial.ext import nj_serial, count


def print_log(path, baudrate, cmd_list):
    '''
        打印串口日志
    @param path:目录文件夹名
    @param cmd_list:串口端口
    @param baudrate:打印波特率
    @return:
    '''
    nj_serial.print_log(path, baudrate, cmd_list)


def conut_reconnect_data(excel_path, excel_sheet, path, key):
    '''
        分析重连异常日志
    @param excel_path:断电excel文件路径
    @param excel_sheet:excel表名称
    @param path:日志目录名
    @param key:在线关键字--> network status:9
    @return:
    '''
    count.analysis(excel_path, excel_sheet, path, key)


def conut_on_data(path, key1, key2, new_path):
    '''
        分析在线日志
    @param path:日志目录路径
    @param key1:在线关键字
    @param key2:离线关键字
    @param new_path:新的文件-->'./test.xlsx'
    @return:
    '''
    count.on_line(path, key1, key2, new_path)


if __name__ == '__main__':
    print_log('通用固件', 256000, ['COM5'])
    print_log('通用固件1', 115200, [])



from . import nj_re

'''
    数据转换模块
'''


def bytes_to_str(bs, encoding="utf-8"):
    '''
        byt转str
    :param bs:byt数据
    :param encoding:读取格式 base64,utf-8
    :return:转换后数据
    '''
    return bs.decode(encoding, 'strict')


def list_bytes_to_hexstr(bs):
    '''
        byt列表转16进制字符串
    :param bs:
    :return:
    '''
    value = ''
    for i in range(len(bs)):
        value += ''.join(['%02X ' % b for b in bs[i]])
    return value


def str_to_hexstr(str):
    '''
        字符串转16进制
    :param str:
    :return:
    '''
    return bytes.fromhex(str)


def hex_to_str(he):
    '''
        0x数据去掉0x
    :param he:16进制0xbc
    :return:去除0x
    '''
    by = nj_re.rm_to_str(str(he), filter='0x')
    if len(by) == 1:
        return "0" + by
    return by


def byt_to_hex(bi):
    '''
        bit转16进制字符串
    :param bi:要转换的bit位列表
    :return:
    '''
    try:
        if len(bi) > 0:  # 当列表为空时返回00
            sum = 0
            for i in bi:
                if i == 0:
                    sum += 1
                else:
                    sum += 2 ** i
            return hex_to_str(hex(sum))
        else:
            return "00"
    except Exception as err:
        print(err)


def data_replace_byt(data, bi):
    '''
        数据更换bit位
    :param data:初始数据
    :param bi:byt列表
    :return:转换后数据
    '''
    list1 = str_to_list(data, " ")
    for i in bi:
        if type(bi[i]) is list:
            list1[i - 1] = byt_to_hex(bi[i])
        else:
            list1[i - 1] = hex_to_str(hex(bi[i]))
    return str_add_space(list_to_str(list1))


def str_add_space(s):
    '''
        字符串+空格-->串口数据
    :param s:初始字符串
    :return:添加空格后字符串
    '''
    result = ''
    for i in range(len(s)):
        result += s[i]
        if i % 2 == 1:
            result += " "
    return result


def hex_to_bytlist(x):
    '''
        16进制转二进制列表
    :param x:16进制数据
    :return:二进制列表
    '''
    bytlist = []
    value = nj_re.rm_to_str(bin(int(x, 16)), "0b")
    for i in range(len(value)):
        if value[i] == '1':
            bytlist.append(len(value) - i)
    return bytlist


def hex_to_byt(cmd, position):
    '''
        16进制转二进制列表
    :param cmd:16进制数据
    :param position:解析位置
    :return:二进制列表
    '''
    return hex_to_bytlist(str_to_list(cmd)[position - 1])


def str_to_list(s, format=" "):
    '''
        字符串转列表
    :param s:初始字符串
    :param format:转换格式
    :return:转换后数据
    '''
    return s.split(format)


def list_to_str(list1):
    '''
        列表转成字符串
    :param list1:列表
    :return:字符串
    '''
    return ''.join(list1)


def checksum(cmd, formula=None, position=None, data_format='s'):
    '''
        效验位公式计算
    :param cmd:初步值
    :param formula:效验公式[BCC,CRC,CRC8,CRC16]
    :param position:效验位
    :param data_format:数据格式->默认小端s
    :return:效验后数据
    '''

    def list_add(value):
        if position == None:
            if len(value) > 2:
                if len(value) == 3:
                    value = "0{}".format(value)
                if data_format == 's':
                    list1.append(value[2:4])
                    list1.append(value[0:2])
                else:
                    list1.append(value[0:2])
                    list1.append(value[2:4])
            else:
                list1.append(value)
        else:
            if len(value) > 2:
                if len(value) == 3:
                    value = "0{}".format(value)
                if data_format == 's':
                    list1.insert(position - 1, value[2:4])
                    list1.insert(position, value[0:2])
                else:
                    list1.insert(position - 1, value[0:2])
                    list1.insert(position, value[2:4])
            else:
                list1.insert(position - 1, value)

    list1 = nj_re.rm_to_list(cmd)
    if formula == "BCC":
        list_add(hex_to_str(serial_crc.bcc(cmd)))
    elif formula == "CRC":
        list_add(hex_to_str(serial_crc.crc(cmd)))
    elif formula == "CRC8":
        list_add(hex_to_str(serial_crc.crc8(cmd)))
    elif formula == "CRC16":
        list_add(hex_to_str(serial_crc.crc16(cmd)))
    return str_add_space(list_to_str(list1))



from njserial.ext.utils import nj_file, nj_excel, nj_time, nj_re


def analysis(excel_path, excel_sheet, path, key):
    '''
        分析日志
    :param excel_path: 断电excel文件路径
    :param excel_sheet: excel表名称
    :param path: 日志目录名
    :param key: 在线关键字
    :return:
    '''
    file_name = nj_file.get_son_file_info(path)  # 获取目录下所有子文件
    print(file_name)
    for name in file_name:
        print(name)
        list1 = []
        list1.extend(nj_file.read_file("{}/{}".format(path, name), key))
        excel = nj_excel.Excel(excel_path)
        ws = excel.wb[excel_sheet]
        rows = ws.rows
        excel.set_col_width(excel_path, name, [12, 12, 12])
        ws = excel.get_sheet(excel_path, name)
        ws.append(['重连时间(S)', '电源开时间', '连上云时间'])
        end = 0
        if len(list1) < 1:
            ws.append(['连云全部失败'])
            excel.end_save(excel_path, name)
            continue
        for row in rows:
            line = [col.value for col in row]
            if line[0] == '开':
                if float(nj_time.count_time(line[1], list1[len(list1) - 1])) < 0:  # 判断excel时间是否大于最大连云时间
                    ws.append(['重连一直失败', line[1], list1[len(list1) - 1]])
                    break
                for i in range(end, len(list1)):
                    times = float(nj_time.count_time(line[1], list1[i]))
                    if times > 0:
                        # print(list1[i])
                        end = i
                        if times < 60:
                            ws.append([times, line[1], nj_time.get_time(list1[i])])
                        else:
                            ws.append(['重连大于60S', line[1], nj_time.get_time(list1[i])])
                        break
        excel.end_save(excel_path, name)


def cont_9600(try_cmd, serial_data_9600list):
    '''
        9600数据分析
    :param try_cmd:查询的数据
    :param serial_data_9600list:日志list
    :return:
    '''
    max = len(serial_data_9600list)
    for i in range(max - 1):
        listdata = "{}{}".format(nj_re.find_rm_second(serial_data_9600list[i]),
                                 nj_re.find_rm_second(serial_data_9600list[i + 1]))  # 去除时间的两组数据合并
        if nj_re.rm_to_str(try_cmd, filter=' ') in nj_re.rm_to_str(listdata, filter=' '):  # 去空格后数据对比
            return i
        elif i == (max - 2):
            return 0


sheet = '在线统计'
excel_path = './test.xlsx'
excel = nj_excel.Excel(excel_path, True)


def on_line(path, key1, key2, new_path):
    global sheet, excel_path, excel
    '''
        115200数据判断在线离线
    :param path:文件目录
    :param key1:在线关键字
    :param key2:离线关键字
    :param excel_path:excel保存路径
    :return:离线结果
    '''
    aa = []
    excel_path = new_path
    excel = nj_excel.Excel(new_path, True)  # 创建Excel
    file_name = nj_file.get_son_file_info(path)  # 获取目录下所有子文件
    for i in range(len(file_name)):
        if i == 0:
            aa.append("{}".format(nj_re.find_data(file_name[i], 'COM\d+'), i))
        else:
            if not "{}".format(nj_re.find_data(file_name[i], 'COM\d+')) in aa:
                aa.append(nj_re.find_data(file_name[i], 'COM\d+'))
    for i in range(len(aa)):  # 遍历当前所有文件
        list1 = []
        list2 = []
        initdata = None  # 包含初始时间数据
        enddata = None  # 包含结束时间数据
        sheet = aa[i]  # 将sheet指向当前端口
        for name in file_name:
            if nj_re.find_data(name, 'COM\d+') == aa[i]:
                list1.extend(nj_file.read_file("{}/{}".format(path, name), key1))
                list2.extend(nj_file.read_file("{}/{}".format(path, name), key2))
                if initdata is None:
                    initdata = nj_file.read_file_count("{}/{}".format(path, name))[0]
                enddata = nj_file.read_file_count("{}/{}".format(path, name))[1]
        on_analysis(list1, list2, initdata, enddata)


def on_analysis(list1, list2, initdata=None, enddata=None):
    '''
        分析日志+结果记录
    :param list1:在线列表
    :param list2:离线列表
    :param initdata:第一行
    :param enddata:最后一行
    :return:
    '''
    ci = 0
    end_time = 0
    end_j = 0
    time_list = []
    excel.set_col_width(excel_path, sheet, [12, 12, 12])
    ws = excel.get_sheet(excel_path, sheet)
    if initdata is None or enddata is None:
        ws['A1'] = '{}-->{}'.format("", "")
    else:
        ws['A1'] = '{}-->{}'.format(nj_time.get_time(initdata), nj_time.get_time(enddata))
    ws.append(['重连耗时(S)', '离线时间', '在线时间'])
    excel.set_row_colour(excel.wb, ws, 2)
    for i in range(len(list2)):
        if end_time == 0 or float(nj_time.count_time(end_time, list2[i])) > 0:  # 离线时间大于最后结束时间
            off = True
            for j in range(end_j, len(list1)):
                times = float(nj_time.count_time(list2[i], list1[j]))
                if times > 0:  # if在线大于离线
                    ci += 1
                    end_time = list1[j]
                    ws.append([times, nj_time.get_time(list2[i]), nj_time.get_time(list1[j])])  # 将离线数据添加到excel
                    time_list.append(times)
                    off = False
                    end_j = j
                    break
            if off:
                ci += 1
                ws.append(["最后一次离线时间：{}".format(nj_time.get_time(list2[i]))])
                break
    if len(time_list) > 0:
        if initdata is None or enddata is None:
            ws['A1'] = '共配网{}次，最大配网时间:{}秒，平均配网时间:{}秒'.format(ci, "%.3f" % float(max(time_list)),
                                                             "%.3f" % float(sum(time_list) / len(time_list)))
        else:
            ws['A1'] = '共离线{}次，总时间：{}秒，总离线:{}秒，最大离线:{}秒，平均离线:{}秒,时间：{}-->{}'.format(ci, nj_time.count_time(initdata,
                                                                                                           enddata),
                                                                                    "%.3f" % float(sum(time_list)),
                                                                                    "%.3f" % float(max(time_list)),
                                                                                    "%.3f" % float(sum(time_list) / len(
                                                                                        time_list)),
                                                                                    nj_time.get_time(initdata),
                                                                                    nj_time.get_time(enddata))
    else:
        ws['A1'] = '共离线{}次，总时间：{}秒，时间：{}-->{}'.format(ci, nj_time.count_time(initdata, enddata),
                                                      nj_time.get_time(initdata), nj_time.get_time(enddata))
    ws.merge_cells('A1:C1')  # 合并单元格
    excel.end_save(excel_path, sheet)



import re
from datetime import datetime

'''
    时间处理模块
'''


def get_time(data=None):
    '''
        获取数据时间
    :param  data 带时间参数的数据,如果为空获取当前时间
    :return:以%m-%d %H:%M:%S.%f格式返回时间
    '''
    time_format = "%m-%d %H:%M:%S.%f"
    if data is None:
        date = datetime.now().strftime(time_format)
    else:
        date = data
    t = re.search(r'{}'.format(r'(\d+)-(\d+) (\d+):(\d+):(\d+).(\d+)'), date)
    end_time = "{}-{} {}:{}:{}.{}".format(t.group(1), t.group(2), t.group(3), t.group(4), t.group(5),
                                          t.group(6)[:-3])
    return end_time


def count_time(init_time_data, end_time_data, year=False):
    '''
        结束数据-开始时间
    :param init_time_data:包含开始时间的数据
    :param end_time_data:包含结束时间的数据
    :return:相减后多少秒
    '''
    time1 = re.search(r'{}'.format(r'(\d+)-(\d+) (\d+):(\d+):(\d+).(\d+)'), init_time_data)  # 获取查找到数据
    time2 = re.search(r'{}'.format(r'(\d+)-(\d+) (\d+):(\d+):(\d+).(\d+)'), end_time_data)  # 获取查找到数据
    print(end_time_data)
    time1s = int(time1.group(3)) * 60 * 60 + int(time1.group(4)) * 60 + int(time1.group(5)) + int(time1.group(3))
    time2s = int(time2.group(3)) * 60 * 60 + int(time2.group(4)) * 60 + int(time2.group(5)) + int(time2.group(3))
    startDay = '{}-{}-{}'.format(str(datetime.now().year), str(time1.group(1)), str(time1.group(2)))
    endDay = '{}-{}-{}'.format(str(datetime.now().year), str(time2.group(1)), str(time2.group(2)))
    if int(time2.group(1)) < int(time1.group(1)) and year:
        startDay = '{}-{}-{}'.format(str(datetime.now().year - 1), str(time1.group(1)), str(time1.group(2)))
    days = (datetime.strptime(endDay, "%Y-%m-%d") - datetime.strptime(startDay, "%Y-%m-%d")).days
    # print(days)
    return "%.3f" % (days * 24 * 60 * 60 + time2s - time1s)


# def year_count_day(year):
#     '''
#         统计当前年有多少天
#     :param year:输入年
#     :return:有多少天
#     '''
#     startDay = str(year) + '-01-01'  # 一年第一天
#     endDay = str(year) + '-12-31'  # 一年最后一天
#
#     # 天数
#     year_days_mum = (datetime.datetime.strptime(endDay, "%Y-%m-%d") - datetime.datetime.strptime(startDay,
#                                                                                                  "%Y-%m-%d")).days + 1
#     return year_days_mum
#
#
# def month_count_day(year, month):
#     '''
#         统计当前月有多少天
#     :param year:输入年
#     :param month:输入月
#     :return:有多少天
#     '''
#     return calendar.monthrange(year, int(month))[1]


if __name__ == '__main__':
    print(get_time())
    # print(count_time("01-01 11:45:09.175  dd dd", "01-01 11:45:9.176  dd dd"))



def check_alive(fn):
    '''
        检查串口是否开启
    @param fn:
    @return:
    '''

    def inner(self, *args, **kwargs):
        if not self.isOpen():  # 判断串口是否连接
            raise NameError('串口未连接,请重试')
        return fn(self, *args, **kwargs)

    return inner



import njserial as ser

# python setup.py bdist --format=wininst  生成exe文件
# pyinstaller -F hello.py 生成exe文件
# pip install --pre --upgrade njserial 更新本地环境

# python setup.py sdist 打包成zip文件发布
# twine upload dist/njserial-1.0.0.tar.gz 上传到pypi服务器

if __name__ == "__main__":
    print(ser.print_log('通用固件', 256000, ['COM5']))



from openpyxl import Workbook, load_workbook
from openpyxl.drawing.image import Image
from openpyxl.styles import PatternFill, Font, Alignment, Border, Side


class Excel(object):

    def __init__(self, excel_path, new=False):
        '''
            创建EXCEL
        :param excel_path:EXCEL目录
        :param new:是否重新创建
        '''
        self.excel_path = excel_path
        if new:
            self.wb = Workbook()
            self.wb.save(excel_path)
        self.wb = load_workbook(excel_path)
        self.MAP = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T']

    def set_col_width(self, excel_path, excel_sheet, col_list=[]):
        '''
            设置列宽度
        :param excel_path:excel目录
        :param excel_sheet:sheet名称
        :param col_list:要改的col列表
        :return:
        '''
        ws = self.get_sheet(excel_path, excel_sheet)
        for i in range(len(col_list)):
            ws.column_dimensions[self.MAP[i]].width = col_list[i] * 2
        if type(excel_path) == str and type(excel_sheet) == str:
            self.wb.save(excel_path)  # 保存文件

    def set_row_colour(self, excel_path, excel_sheet, row=None, colour='E1FFFF'):
        '''
            为单元格填充颜色
        :param excel_path:excel目录
        :param excel_sheet:sheet名称
        :param row:第几行
        :param colour:颜色值
        :return:
        '''
        fill = PatternFill(patternType="solid", start_color=colour)
        ws = self.get_sheet(excel_path, excel_sheet)
        if row is None:
            row = ws.max_row
        for i in range(ws.max_column):
            ws['{}{}'.format(self.MAP[i], row)].fill = fill
        if type(excel_path) == str and type(excel_sheet) == str:
            self.wb.save(excel_path)  # 保存文件

    def get_end_value(self, excel_path, excel_sheet, max_line=None):
        '''
            整行读取值
        :param excel_path:excel目录
        :param excel_sheet:sheet名称
        :param max_line:查找行
        :return:
        '''
        ws = self.get_sheet(excel_path, excel_sheet)
        list1 = []
        rows = ws.rows
        for row in rows:
            line = [col.value for col in row]
            list1.append(line)
        if max_line is None:
            max_line = ws.max_row - 1
        return list1[max_line]

    def insert_image(self, excel_path, excel_sheet, image_path, col, row=None):
        '''
            插入图片
        :param excel_path:excel目录
        :param excel_sheet:sheet名称
        :param image_path:image目录
        :param col:列
        :param row:行
        :return:
        '''
        ws = self.get_sheet(self.excel_path, excel_sheet)
        ws.column_dimensions[self.MAP[col]].width = 13  # 修改列宽
        if row is None:
            row = ws.max_row
        ws.row_dimensions[row].height = 36  # 修改行高
        img = Image(image_path)  # 获取图像
        img.width, img.height = 100, 50  # 图片设置宽高
        ws.add_image(img, '{}{}'.format(self.MAP[col], row))  # 插入图片
        if type(excel_path) == str and type(excel_sheet) == str:
            self.wb.save(excel_path)  # 保存文件

    def add_value(self, excel_path, excel_sheet, list_value):
        '''
            添加一列值
        :param excel_path:excel路径
        :param excel_sheet:sheet名称
        :param list_value:插入列表值
        :return:
        '''
        ws = self.get_sheet(self.excel_path, excel_sheet)
        ws.append(list_value)
        self.wb.save(excel_path)

    def get_sheet(self, excel_path, excel_sheet):
        '''
            获取Sheet
        :param self.wb:打开的excel
        :param excel_sheet:sheet名称
        :return:ws
        '''
        if type(excel_path) == str and type(excel_sheet) == str:
            self.wb = load_workbook(excel_path)
        else:
            self.wb = excel_path
        if len(self.wb.sheetnames) < 2 and self.wb.sheetnames[0] == "Sheet":
            ws = self.wb.active
            ws.title = excel_sheet
        else:
            if type(excel_sheet) == str:
                if not excel_sheet in self.wb.sheetnames:  # 遍历所有sheet
                    self.wb.create_sheet(title=excel_sheet)
                ws = self.wb[excel_sheet]
            else:
                ws = excel_sheet
        return ws

    def end_save(self, excel_path, excel_sheet):
        '''
            最后设置整体EXCEL格式
        :param excel_path:excel目录
        :return:
        '''
        self.wb.save(excel_path)  # 先保存之前数据
        ws = self.get_sheet(excel_path, excel_sheet)
        font = Font(size=11, bold=False, name='微软雅黑', color="000000")
        thin = Side(border_style="thin", color="000000")
        border = Border(left=thin, right=thin, top=thin, bottom=thin)
        alignment = Alignment(horizontal="left", vertical="center", wrap_text=True)  # wrap_text=True 打开自动换行
        max_row = ws.max_row
        max_col = ws.max_column
        for row in ws['A1:{}{}'.format(self.MAP[max_col - 1], max_row)]:
            for cell in row:
                cell.font = font  # 修改文字颜色
                cell.border = border  # 加边框
                cell.alignment = alignment  # 修改格式
                if cell.value == "Fail" or cell.value == "fail":
                    cell.fill = PatternFill(patternType="solid", start_color="FF0000")
        for i in range(ws.max_row + 1):
            ws.row_dimensions[i].height = 36
        self.wb.save(excel_path)

    def other(self, ws):
        '''
            没用上资料备份
        :param ws: 打开的sheet
        :return:
        '''
        ws.sheet_properties.tabColor = "1072BA"  # 为sheet底部加颜色
        ws.append(['aa', 'ss'])  # 写入整column
        rows = ws.rows  # 读取整row
        columns = ws.columns  # 读取整column
        rows = ws.max_row  # 获取最大行数
        cols = ws.max_column  # 获取最大列数
        ws['A4'] = 4  # 单行写入
        print(ws['A4'].value)  # 单行读取
        ws.column_dimensions['A'].width = 13  # 修改列宽
        ws.row_dimensions[1].height = 36  # 修改行高
        # 合并单元格
        ws.merge_cells('A1:G1')
        ws.save(path)


if __name__ == "__main__":
    path = r'./excel.xlsx'
    excel = Excel(path, True)
    ws = excel.get_sheet(path, '计划')
    # excel.insert_image(path, '计划', './1.png', 'A', 3)
    # print(excel.get_send_value(path, '计划'))
    ws['B5'] = "Fail"
    excel.wb.save(path)
    excel.set_col_width(path, '计划', [16, 17, 16])
    excel.end_save(path, '计划')



import threading
import time
from serial import Serial
from njserial.ext.utils import nj_parse, nj_time, nj_file
from njserial.utils import check_alive
import serial.tools.list_ports

'''
    串口通讯模块
'''


class NjSerial(Serial):
    #     '''
    #         连接串口
    #     :param port:端口号
    #     :param baudrate:波特率(普通模块:9600,115200，5981:256000)
    #     :param bytesize:数据位
    #     :param parity:效验位('N', 'E', 'O', 'M', 'S')
    #     :param stopbits:停止位
    #     '''

    @check_alive
    def start_log(self):
        self.state = True
        self.thread_read = threading.Thread(target=self.start_read_cmd, args=())
        self.thread_read.start()
        if self.baudrate == 9600:
            self.thread_read_9600 = threading.Thread(target=self.start_9600, args=())
            self.thread_read_9600.start()

    @check_alive
    def start_read_cmd(self):
        '''
            开启读取串口数据
        :return:
        '''
        self.tips = 1
        self.serial_data = []
        self.serial_data_115200list = []
        while self.state:
            try:
                if self.isOpen():
                    if self.baudrate != 9600:
                        data = self.readline()
                        if data != b'':
                            content = nj_parse.bytes_to_str(data, encoding="utf-8")
                            datas = content.replace('\n', '').replace('\r', '')  # 去除换行
                            self.serial_data_115200list.append(add_time(datas))  # 添加115200打印数据
                    else:
                        data = self.read()
                        self.serial_data.append(data)
                        self.start_time = time.time()
                        # print(nj_parse.bytes_to_hexstr(data)) # 查看9600单个数据
                    if self.tips > 0:
                        self.tips -= 1
                        print("{} ：{}".format(self.port, add_time(data)))  # 刚启动时打印数据
            except Exception as err:
                print("读取串口数据报错" + str(err))

    @check_alive
    def start_9600(self):
        '''
            9600循环数据打印
        @return:
        '''
        self.serial_data_9600list = []
        while self.state:
            if (time.time() - self.start_time) >= 0.06 and len(self.serial_data) > 4:
                self.content = nj_parse.list_bytes_to_hexstr(self.serial_data)
                self.serial_data = []
                self.start_time = time.time()
                self.serial_data_9600list.append(add_time(self.content))
                # print("结果：" + self.add_time(self.content))

    @check_alive
    def stop_serial(self):
        '''
            关闭串口连接
        :return:
        '''
        self.state = False
        time.sleep(0.5)
        self.close()


def add_time(data, value=''):
    '''
        为数据增加时间节点
    :param data:初始数据
    :param value:中间要插入的数据
    :return:合并后数据
    '''
    return "{} {}{}".format(nj_time.get_time(), value, data)


def print_log(path, baudrate, cmd_list):
    '''
        打印串口日志
    @param path:目录文件夹名
    @param baudrate:波特率
    @param cmd_list:串口端口
    @return:
    '''
    nj_file.path_exists(path)  # 判断文件夹
    print(get_port())  # 打印当前所有端口
    if len(cmd_list) > 0:
        for cmd in cmd_list:
            threading.Thread(target=keep_data, args=(path, cmd, baudrate, False)).start()
    else:
        time.sleep(1)
        port_list = check_port()  # 检查可用端口
        if not len(port_list) > 0:
            raise NameError('目前无串口可用,请插入新串口')
        for cmd in port_list:
            threading.Thread(target=keep_data, args=(path, cmd, baudrate, True)).start()
        time.sleep(5)
        print("{}-->新增串口：{}".format(path, str(PORT)))


PORT = []


def keep_data(path, cmd, baudrate=115200, prints=False):
    global PORT
    '''
        保存打印数据
    :param path: 文件保存目录
    :param cmd:端口号
    :param prints:判断是否打印正连接的端口号
    :return:
    '''
    file_path = "{}/{}日志1.log".format(path, cmd)
    try:
        ser = NjSerial(cmd, baudrate)
        ser.start_log()
        while True:
            nj_file.file_add_value(file_path)  # 增加开始打印时间
            time.sleep(4)
            if len(ser.serial_data_115200list) > 0:
                if prints:  # 判断初始为空端口
                    PORT.append(cmd)
                nj_file.add_list(file_path, ser.serial_data_115200list)  # 保存数据
                file_path = nj_file.judge_file_size(file_path)  # 判断文件大小
            time.sleep(27)
    except Exception as err:
        print("串口日志报错：" + str(err))


def check_port():
    '''
        打印可用端口
    :return:返回可用端口列表
    '''
    serial_list = []
    for i in get_port():
        try:
            ser = NjSerial(port=i)
            ser.close()
            serial_list.append(i)
        except Exception as err:
            pass
    return serial_list


def get_port():
    '''
        获取所有串口号
    :return: 所有串口的名字-->如‘com3’
    '''
    all_serial_list = show_all_serial()
    usb_serial_list = []
    for i in all_serial_list:
        if 'USB Serial Port' in i[1]:
            usb_serial_list.append(i[0])
    return usb_serial_list


def show_all_serial():
    '''
        获取所有的串口信息
    :return:所有的串口及每个窜口的信息
    '''
    plist = list(serial.tools.list_ports.comports())  # 获取串口信息
    all_serial_list = []
    for p in plist:
        serial_info = []
        for i in p:
            serial_info.append(i)
        all_serial_list.append(tuple(serial_info))
    return all_serial_list



import os
import re
import zipfile

from njserial.ext.utils import nj_time

'''
    文件处理模块
    字母存储：ISO-8859-1
'''


def path_exists(path):
    '''
        创建文件夹
    :param path:要创建的文件夹
    :return:
    '''
    if not os.path.exists(path):
        os.makedirs(path)


def get_son_file_info(file_dir, info='files'):
    '''
        获取文件信息
    :param file_dir:当前目录
    :return:
    '''
    for root, dirs, files in os.walk(file_dir):
        if info == 'files':  # 所有非目录子文件
            return files
        if info == 'root':  # 目录路径
            return root
        if info == 'dirs':  # 所有子目录
            return dirs


def mv_file(old_name, new_name):
    '''
        更换文件名
    :param old_name:旧文件名
    :param new_name:新文件名
    :return:
    '''
    os.rename(old_name, new_name)


def judge_file_size(path):
    '''
        判断当前文件大小-如大于50M将文件名称最后位+1
    :param path:文件目录
    :return:修改后文件
    '''
    size = get_doc_size(path)
    if not 'kb' in size:
        if int(re.match(r'\d+', size).group()) < 50 and 'M' in size:
            return path
        else:
            old = re.search(r'{}+(\d+)\.'.format('[\u4e00-\u9fa5]'), path)
            new = re.sub(old.group(1), str(int(old.group(1)) + 1), old.group())
            return re.sub(old.group(), new, path)
    else:
        return path


def add_list(path, list_data, line=True):
    '''
        向文件中写入数据
    :param path:文件目录
    :param list_data:list数据
    :return:成功
    '''
    with open(path, 'a+', encoding="utf-8")as fd:
        count = fd.tell()
        fd.seek(count, 0)
        for i in range(len(list_data)):
            fd.write(list_data[i])
            if line:
                fd.write('\n')
        list_data.clear()  # 插入完清除数据
        fd.close()
        return True


def get_file_name(path):
    '''
        获取地址中文件名称
    :param path:文件地址
    :return:列表[文件名,后缀]
    '''
    if re.search(r'/', path) != None:
        if re.search(r'\.', path) != None:
            a = re.search(r'/(.+)\.(.+)', path)
            return [a.group(1), a.group(2)]
        else:
            return [re.search(r'/(.+)', path).group(1)]
    else:
        if re.search(r'\.', path) != None:
            a = re.search(r'(.+)\.(.+)', path)
            return [a.group(1), a.group(2)]
        else:
            return [path]


def file_add_value(path):
    '''
        向文件第一行中加入数据
    :param path:文件地址
    :return:
    '''
    with open(path, 'a+', encoding="utf-8")as fd:
        count = fd.tell()
        if count == 0:
            fd.write("{} 开始打印数据".format(nj_time.get_time()))
            fd.write('\n')
        fd.close()


def zipDir(dirpath, outFullName):
    """
        压缩指定文件夹
    :param dirpath: 目标文件夹路径
    :param outFullName: 压缩文件保存路径+xxxx.zip
    :return: 无
    """
    zip = zipfile.ZipFile(outFullName, "w", zipfile.ZIP_DEFLATED)
    for path, dirnames, filenames in os.walk(dirpath):
        fpath = path.replace(dirpath, '')  # 去掉目标跟路径，只对目标文件夹下边的文件及文件夹进行压缩
        for filename in filenames:
            zip.write(os.path.join(path, filename), os.path.join(fpath, filename))
    zip.close()


def get_doc_size(path):
    '''
        获取文件大小
    :param path:文件目录
    :return:文件大小
    '''
    size = os.path.getsize(path)
    return formatSize(size)


def get_file_size(path):
    '''
        获取文件夹大小
    :param path: 文件夹目录
    :return:文件夹大小
    '''
    sumsize = 0
    try:
        filename = os.walk(path)
        for root, dirs, files in filename:
            for fle in files:
                size = os.path.getsize(path + fle)
                sumsize += size
        return formatSize(sumsize)
    except Exception as err:
        print(err)


def read_file(path, keyword, count=0):
    '''
        读取文件
    :param path:文件目录
    :param count:指针指向位置
    :return:包含关键字[]
    '''
    with open(path, 'r', encoding="utf-8")as fd:
        fd.seek(count, 0)  # 指针指向当前count位置
        list1 = []
        while True:
            line = fd.readline()
            if not line:
                fd.close()
                break
            if keyword in line:
                list1.append(line)
        fd.close()
        return list1


def read_file_count(path):
    '''
        读取文件第一行和最后一行
    :param path:文件地址
    :return:第一行和最后一行列表
    '''
    with open(path, 'rb')as fd:
        first_line = fd.readline()  # 取第一行
        offset = -50  # 设置偏移量
        while True:
            fd.seek(offset, 2)  # 指针指向当前offset位置 , whence=2从末尾开始算
            lines = fd.readlines()
            if len(lines) >= 2:  # 判断是否最后至少有两行，这样保证了最后一行是完整的
                last_line = lines[-1]  # 取最后一行
                break
            offset *= 2
        fd.close()
        return [first_line.decode(), last_line.decode()]


def formatSize(bytes):
    '''
        字节bytes转化kb\m\g
    :param bytes:
    :return:
    '''
    try:
        bytes = float(bytes)
        kb = bytes / 1024
    except:
        print("传入的字节格式不对")
        return "Error"

    if kb >= 1024:
        M = kb / 1024
        if M >= 1024:
            G = M / 1024
            return "%.0fG" % (G)
        else:
            return "%.0fM" % (M)
    else:
        return "%.0fkb" % (kb)



import re

'''
    正则匹配模块
    match : 从第一个字段开始匹配
    search：全局搜索匹配
    sub：替换数据
    split 分割字符串-返回列表
    findall：查找所有符合的数据-返回列表
    r' ' 表示原⽣字符串
    \d ：匹配数字，\D：匹配非数字
    \w ：匹配单字符串  \W：匹配⾮单词字符
    [4-7]：匹配括号中字符
    \：\\\\匹配
    ()：分组匹配-->r.group(1)
    + 出现1次或者⽆限次
    ? 出现1次或者0次--在最后一个值前加可关闭贪婪模式
    {m} 连续出现m次
    {m,} 前⼀个字符出现m次-->至少有多少个相连接
    {m,n} 出现从m到n次
    ^ 匹配字符串开头
    $ 匹配字符串结尾
    匹配. --> \.
    匹配中文 -->'[\u4e00-\u9fa5]'
'''


def rm_to_str(data, filter=' '):
    '''
       分割字符串返回字符串
    :param data:初始数据
    :param filter:分割条件(分割后去除)
    :return:分割后列表
    '''
    return re.sub(r"{}".format(filter), '', data)


def rm_to_list(data, filter=' '):
    '''
       分割字符串返回list
    :param data:初始数据
    :param filter:分割条件(分割后去除)
    :return:分割后列表
    '''
    return re.split(r"{}".format(filter), data)


def find_rm_match(data, filter='.+\.'):
    '''
        查找删除数据（从开头）--初始：提取毫秒时间
    :param data:初始数据
    :param filter:匹配条件
    :return:删除匹配条件后数据
    '''
    a = re.match(r'{}'.format(filter), data).group()  # 获取查找到数据
    return rm_to_str(data, a)


def find_rm_search(data, filter='.+\.'):
    '''
        查找删除数据（从中间匹配）
    :param data:初始数据
    :param filter:匹配条件
    :return:删除匹配条件后数据
    '''
    a = re.search(r'{}'.format(filter), data).group()  # 获取查找到数据
    return rm_to_str(data, a)


def find_rm_second(data, second=2, filter='.+? '):
    '''
        查找删除匹配格式出现次数数据
    :param data:初始数据
    :param second:第几次前删除
    :param filter:查找格式
    :return:删除后数据
    '''
    str1 = find_data_second(data, second, filter)
    return rm_to_str(data, str1)


def find_data_second(data, second=2, filter='.+? '):
    '''
        查找匹配格式出现次数数据
    :param data:初始数据
    :param second:截取出现次数前数据
    :param filter:过滤条件
    :return:查找到的数据
    '''
    list1 = re.findall(r'{}'.format(filter), data)  # 获取查找到数据
    str1 = ''
    for i in range(second):
        str1 += list1[i]
    return str1


def find_add_match(data, data1, filter='.+\.'):
    '''
        查找添加数据（从开头）
    :param data:初始数据
    :param data1:要添加的数据
    :param filter:过滤条件
    :return:添加后数据
    '''
    a = re.match(r'{}'.format(filter), data).group()  # 获取查找到数据
    return "{}{}".format(a, data1)


def find_add_search(data, data1, filter='.+\.'):
    '''
        查找添加数据（从中间匹配）
    :param data:初始数据
    :param data1:要添加的数据
    :param filter:过滤条件
    :return:添加后数据
    '''
    a = re.search(r'{}'.format(filter), data).group()  # 获取查找到数据
    return "{}{}".format(a, data1)


def find_data(data, filter):
    '''
        按格式截取数据
    :param data:初始数据
    :param filter:截取条件
    :return:截取到的数据
    '''
    return re.search(r'{}'.format(filter), data).group()


def find_mid_search(data, filter='{', filter1='}'):
    '''
        查找截取中间值
    :param data:初始数据
    :param filter:过滤初
    :param filter1:过滤尾
    :return:从过滤初到过滤尾查的数据
    '''
    r = re.search(r'({}.+{})'.format(filter, filter1), data)  # 获取查找到数据
    return r.group(1)



from setuptools import setup

setup(
    name="njserial",  # 这里是pip项目发布的名称
    version="1.0.1",  # 版本号，数值大的会优先被pip
    keywords=("pip", "SICA", "featureextraction"),
    description="Serial Tool",  # 平台提示
    long_description="Automated Packaging Tool",
    license="MIT Licence",

    url="https://github.com/congrenya/njtest/blob/master/njtest.py",  # 项目相关文件地址，一般是github
    author="Stephen Yao",
    author_email="a1123547734@163.com",

    packages=['njserial', 'njserial.cli', 'njserial.ext', 'njserial.ext.utils'],
    # 打包文件夹
    include_package_data=True,  # 自动打包文件夹内所有数据
    zip_safe=True,  # 设定项目包为安全，不用每次都检测其安全性
    platforms="any",
    install_requires=['pyserial', 'openpyxl']  # 这个项目需要的第三方库
)
